@@ -189,7 +189,10 @@ def gather_pseudovar_props(pseudovars:list[str]):
             output_dict['line_ymax'] = np.nanmax([output_dict['line_ymax'], props['ymax']])
     return output_dict
 
-def tplot(variables, var_label=None,
+
+def tplot(variables,
+          trange=None,
+          var_label=None,
           xsize=None,
           ysize=None,
           save_png='',
@@ -457,7 +460,9 @@ def tplot(variables, var_label=None,
                     pseudo_show_colorbar=True
                 else:
                     pseudo_show_colorbar=False
-                tplot(var, return_plot_objects=return_plot_objects,
+                tplot(var,
+                      trange=trange,
+                      return_plot_objects=return_plot_objects,
                       xsize=xsize, ysize=ysize,
                       fig=fig, axis=this_axis, display=False,
                       running_trace_count=traces_processed,
@@ -484,11 +489,23 @@ def tplot(variables, var_label=None,
             if pytplot.tplot_opt_glob['data_gap'] is not None and pytplot.tplot_opt_glob['data_gap'] > 0:
                 var_data = pytplot.makegap(var_data, dt = pytplot.tplot_opt_glob['data_gap'])
 
-        # set the x-axis range, if it was set with xlim or tlimit
-        if pytplot.tplot_opt_glob.get('x_range') is not None:
-            x_range = pytplot.tplot_opt_glob['x_range']  # Seconds since epoch
-            x_range_start = x_range[0]
-            x_range_stop = x_range[1]
+        # set the x-axis range, if it was set with xlim or tlimit or the trange parameter
+        if trange is None and pytplot.tplot_opt_glob.get('x_range') is None:
+            var_data_times = var_data.times
+            time_idxs = np.arange(len(var_data_times))
+        else:
+            if trange is not None:
+                if len(trange) != 2:
+                    logging.error('Invalid trange setting: must be a 2-element list or array')
+                    return
+                if isinstance(trange[0], str):
+                    x_range = pytplot.time_double(trange) # seconds since epoch
+                    x_range_start = x_range[0]
+                    x_range_stop = x_range[1]
+            else:
+                x_range = pytplot.tplot_opt_glob['x_range']  # Seconds since epoch
+                x_range_start = x_range[0]
+                x_range_stop = x_range[1]
 
             # Check for NaN or inf in x_range
             if not np.isfinite(x_range_start):
@@ -507,9 +524,6 @@ def tplot(variables, var_label=None,
                 logging.info('No data found in the time range: ' + variable)
                 continue
             var_data_times = var_data.times[time_idxs]
-        else:
-            var_data_times = var_data.times
-            time_idxs = np.arange(len(var_data_times))
 
         var_times = var_data_times
 
@@ -559,6 +573,8 @@ def tplot(variables, var_label=None,
         else:
             xtitle_color = None
 
+        if xaxis_options.get('axis_color') is not None:
+            xtitle_color = xaxis_options['axis_color']
 
         ylog = yaxis_options['y_axis_type']
 
@@ -659,7 +675,7 @@ def tplot(variables, var_label=None,
             this_axis.tick_params(axis='y', width=plot_extras.get('ytick_width'))
 
         if plot_extras.get('xtick_labelcolor') is not None:
-            this_axis.tick_params(axis='y', labelcolor=plot_extras.get('xtick_labelcolor'))
+            this_axis.tick_params(axis='x', labelcolor=plot_extras.get('xtick_labelcolor'))
 
         if plot_extras.get('ytick_labelcolor') is not None:
             this_axis.tick_params(axis='y', labelcolor=plot_extras.get('ytick_labelcolor'))
@@ -712,8 +728,8 @@ def tplot(variables, var_label=None,
                                   **hightlight_opts)
 
         # add annotations
-        if pytplot.tplot_opt_glob.get('annotations') is not None:
-            annotations = pytplot.tplot_opt_glob['annotations']
+        if pytplot.data_quants[variable].attrs['plot_options']['extras'].get('annotations') is not None:
+            annotations = pytplot.data_quants[variable].attrs['plot_options']['extras']['annotations']
             for annotation in annotations:
                 this_axis.annotate(annotation['text'], annotation['position'],
                                    xycoords=annotation['xycoords'],
@@ -739,16 +755,12 @@ def tplot(variables, var_label=None,
         if pytplot.data_quants.get(variable) is None:
             continue
         plot_extras = pytplot.data_quants[variable].attrs['plot_options']['extras']
-
+        zaxis_options = pytplot.data_quants[variable].attrs['plot_options']['zaxis_opt']
         if plot_extras.get('spec') is not None:
             spec = plot_extras['spec']
         else:
             spec = False
 
-        if plot_extras.get('colormap_width') is not None:
-            colormap_width = plot_extras['colormap_width']
-        else:
-            colormap_width = 0.02
 
         if spec and show_colorbar:
             if colorbars.get(variable) is None:
@@ -773,20 +785,6 @@ def tplot(variables, var_label=None,
 
             colorbar = fig.colorbar(colorbars[variable]['im'], ax=this_axis)
 
-            """
-            box = this_axis.get_position()
-            pad, width = 0.02, colormap_width
-            cax = fig.add_axes([box.xmax + pad + second_axis_size, box.ymin, width, box.height])
-            if colorbars[variable]['axis_font_size'] is not None:
-                cax.tick_params(labelsize=colorbars[variable]['axis_font_size'])
-            colorbar = fig.colorbar(colorbars[variable]['im'], cax=cax)
-            """
-            """
-            divider = make_axes_locatable(this_axis)
-            cax = divider.append_axes("right", size="5%", pad=0.1 + second_axis_size)
-            print("Adding colorbar for ",variable)
-            colorbar = fig.colorbar(colorbars[variable]['im'], cax=cax)
-            """
             if style is None:
                 ztitle_color = 'black'
             else:
@@ -990,8 +988,12 @@ def mouse_move_slice(event, slice_axes, slice_plot):
 
 def get_var_label_ticks(var_xr, times):
     out_ticks = []
+    extras = var_xr.attrs['plot_options']['extras']
+    var_label_format = extras.get('var_label_format')
+    if var_label_format is None:
+        var_label_format = '{:.2f}'
     for time in times:
-        out_ticks.append('{:.2f}'.format(var_xr.interp(coords={'time': time}, kwargs={'fill_value': 'extrapolate', 'bounds_error': False}).values))
+        out_ticks.append(var_label_format.format(var_xr.interp(coords={'time': time}, kwargs={'fill_value': 'extrapolate', 'bounds_error': False}).values))
     return out_ticks
 
 
