@@ -41,12 +41,14 @@ def options(name, option=None, value=None, opt_dict=None, quiet=False):
         line_width              flt          Sets plot line width.
         line_style              str          scatter (to make scatter plots), or solid_line, dot, dash, dash_dot, dash_dot_dot_dot, long_dash.
         border                  bool         Turns on or off the top/right axes that would create a box around the plot.
-        var_label_ticks         int          Sets the number of ticks if this variable is displayed as an alternative x axis.
-        char_size               int          Defines character size for plot labels, etc.
+        var_label_format        str          The format of the tick labels if this variable is displayed as an alternative x axis. Default: {:.2f}
+        char_size               int          Defines character size for x/y/z titles and subtitles
         right_axis              bool         If true,display a second Y axis on the right side of the plot.
         second_axis_size        numeric      The size of the second axis to display
         data_gap                numeric      If there is a gap in the data larger than this number in seconds, then insert
+        annotations             dict         A dictionary or list of dictionaries of matplotlib text annotation parameters (see annotate() routine)
         visible                 bool         If False, do not display lines for this variable.
+        nodata                  bool         If True, do not display lines for this variable.
         (cont)                  (cont)       NaNs. This is similar to using the degap procedure on the variable, but is
         (cont)                  (cont)       applied at plot-time, and does not persist in the variable data.
         ======================  ===========  ===========================================================================================================================
@@ -55,17 +57,17 @@ def options(name, option=None, value=None, opt_dict=None, quiet=False):
         Legend Options          Value type   Notes
         ======================  ===========  ===========================================================================================================================
         legend_names            list         A list of strings that will be used to identify the legends.
-        legend_location         str         A string giving the location of the legend box.
+        legend_location         str          A string giving the location of the legend box.
         legend_size             numeric      The font size of the legend names
         legend_shadow           bool         Turns on or off drop shadows on the legend box
         legend_title            str          The title to display on the legend
         legend_titlesize        numeric      The font size of the legend title
         legend_color            [str]        The color of the legend names
-        legend_edgecolor        str          The edge color of markers displayed in the legend
-        legend_facecolor        str          The face color of markers displayed in the legend
-        legend_markerfirst      boolean      Put the marker to the left of the line in the legend
+        legend_edgecolor        str          The border color of the legend box
+        legend_facecolor        str          The background color of the legend box
+        legend_markerfirst      boolean      Put the marker and line to the left of the label in the legend
         legend_markerscale      numeric      The scale size of markers displayed in the legend
-        legend_markersize       numeric      The size of the markers displayed in the legend
+        legend_linewidth        numeric      The width of the lines displayed in the legend
         ======================  ===========  ===========================================================================================================================
 
         ======================  ===========  ===========================================================================================================================
@@ -78,7 +80,7 @@ def options(name, option=None, value=None, opt_dict=None, quiet=False):
         xtick_width             numeric      The width of the x tick marks
         xtick_color             str          The color of the x tick marks
         xtick_labelcolor        str          The color of the x tick marks
-        xtick_direction         numeric      The direction of the x tick marks
+        xtick_direction         str          The direction of the x tick marks (in, out, inout)
         ======================  ===========  ===========================================================================================================================
 
 
@@ -94,7 +96,7 @@ def options(name, option=None, value=None, opt_dict=None, quiet=False):
         ytick_width             numeric      The width of the Y tick marks
         ytick_color             str          The color of the Y tick marks
         ytick_labelcolor        str          The color of the Y tick marks
-        ytick_direction         numeric      The direction of the Y tick marks
+        ytick_direction         str          The direction of the Y tick marks (in, out, inout)
         y_major_ticks           [numeric]    A list of values that will be used to set the major ticks on the Y axis.
         y_minor_tick_interval   numeric      The interval between minor ticks on the Y axis.
         ======================  ===========  ===========================================================================================================================
@@ -122,7 +124,6 @@ def options(name, option=None, value=None, opt_dict=None, quiet=False):
         ======================  ===========  ===========================================================================================================================
         spec                    bool         Display this variable as a spectrogram.
         colormap                str/list     Color map to use for specplots https://matplotlib.org/examples/color/colormaps_reference.html.
-        colormap_width          numeric      The width of the specplot color bar
         z_range                 flt/list     Two numbers that give the z axis range of the plot. If a third argument is present, set linear or log scaling accordingly.
         zlog                    int          True sets the z axis to log scale, False reverts (spectrograms only).
         ztitle                  str          Title shown on the z axis. Spec plots only. Use backslash for new lines.
@@ -164,11 +165,10 @@ def options(name, option=None, value=None, opt_dict=None, quiet=False):
         legend_facecolor        label_facecolor, labels_facecolor
         legend_markerfirst      label_markerfirst, labels_markerfirst
         legend_markerscale      label_markerscale, labels_markerscale
-        legend_markersize       label_markersize, labels_markersize
+        legend_linewidth        label_linewidth, labels_linewidth
         legend_frameon          label_frameon, labels_frameon
         legend_ncols            label_ncols, labels_ncols
         line_style_name         line_style, linestyle
-        visible                 nodata
         char_size               charsize
         marker                  markers
         marker_size             markersize
@@ -179,6 +179,8 @@ def options(name, option=None, value=None, opt_dict=None, quiet=False):
         z_range                 zrange
         data_gap                datagap
         spec_dim_to_plot        spec_plot_dim
+        var_label_format        varlabel_format
+        annotations             annotation
         ======================  ======================================================================================================================================
 
 
@@ -248,15 +250,17 @@ def options(name, option=None, value=None, opt_dict=None, quiet=False):
                 if isinstance(value, list):
                     pytplot.link(i, value[1], value[0])
 
+            elif option in ['annotation', 'annotations']:
+                # It is probably more convenient to use the annotations() wrapper function
+                # to manage annotations
+                pytplot.data_quants[i].attrs['plot_options']['extras']['annotations'] = value
+
             elif option == 'colormap':
                 if isinstance(value, list):
                     pytplot.data_quants[i].attrs['plot_options']['extras']['colormap'] = value
                 else:
                     pytplot.data_quants[i].attrs['plot_options']['extras']['colormap'] = [value]
 
-            elif option == 'colormap_width':
-                pytplot.data_quants[i].attrs['plot_options']['extras']['colormap_width'] = value
-
             elif option == 'second_axis_size':
                 pytplot.data_quants[i].attrs['plot_options']['extras']['second_axis_size'] = value
 
@@ -319,8 +323,8 @@ def options(name, option=None, value=None, opt_dict=None, quiet=False):
             elif option in ['legend_markerscale', 'label_markerscale', 'labels_markerscale']:
                 pytplot.data_quants[i].attrs['plot_options']['yaxis_opt']['legend_markerscale'] = value
 
-            elif option in ['legend_markersize', 'label_markersize', 'labels_markersize']:
-                pytplot.data_quants[i].attrs['plot_options']['yaxis_opt']['legend_markersize'] = value
+            elif option in ['legend_linewidth', 'label_linewidth', 'labels_linewidth']:
+                pytplot.data_quants[i].attrs['plot_options']['yaxis_opt']['legend_linewidth'] = value
 
             elif option in ['legend_frameon', 'label_frameon', 'labels_frameon']:
                 pytplot.data_quants[i].attrs['plot_options']['yaxis_opt']['legend_frameon'] = value
@@ -358,8 +362,11 @@ def options(name, option=None, value=None, opt_dict=None, quiet=False):
                 else:
                     pytplot.data_quants[i].attrs['plot_options']['zaxis_opt']['z_axis_type'] = 'linear'
 
-            elif option in ['nodata', 'visible']:
-                pytplot.data_quants[i].attrs['plot_options']['line_opt']['visible'] = value
+            elif option =='visible':
+                pytplot.data_quants[i].attrs['plot_options']['line_opt']['visible'] = bool(value)
+
+            elif option =='nodata':
+                pytplot.data_quants[i].attrs['plot_options']['line_opt']['visible'] = not bool(value)
 
             # Obsolete? (except for value='none'?) JWL 2024-03-21
             # These don't seem to be the correct format for matplotlib parameterized line styles.
@@ -391,6 +398,9 @@ def options(name, option=None, value=None, opt_dict=None, quiet=False):
             elif option in ['char_size', 'charsize']:
                 pytplot.data_quants[i].attrs['plot_options']['extras']['char_size'] = value
 
+            elif option in ['var_label_format', 'varlabel_format']:
+                pytplot.data_quants[i].attrs['plot_options']['extras']['var_label_format'] = value
+
             elif option in ['name', 'title']:
                 pytplot.data_quants[i].attrs['plot_options']['line_opt']['title'] = value
 
