@@ -1,8 +1,3 @@
-# Copyright 2018 Regents of the University of Colorado. All Rights Reserved.
-# Released under the MIT license.
-# This software was developed at the University of Colorado's Laboratory for Atmospheric and Space Physics.
-# Verify current version before use at: https://github.com/MAVENSDC/Pytplot
-
 from .add_across import add_across
 from .add import add
 from .avg_res_data import avg_res_data
@@ -12,12 +7,10 @@ from .deflag import deflag
 from .degap import degap
 from .derive import derive
 from .divide import divide
-from .flatten import flatten
 from .interp_nan import interp_nan
 from .tinterp import tinterp
 from .join_vec import join_vec
 from .multiply import multiply
-from .resample import resample
 from .spec_mult import spec_mult
 from .split_vec import split_vec
 from .subtract import subtract
@@ -29,4 +22,11 @@ from .tcrossp import tcrossp
 from .tnormalize import tnormalize
 from .subtract_median import subtract_median
 from .subtract_average import subtract_average
-from .time_clip import time_clip
\ No newline at end of file
+from .time_clip import time_clip
+from .tkm2re import tkm2re
+from .makegap import makegap
+from .tdeflag import tdeflag
+from .pwrspc import pwrspc
+from .tpwrspc import tpwrspc
+from .dpwrspc import dpwrspc
+from .tdpwrspc import tdpwrspc
\ No newline at end of file
