@@ -699,67 +699,3 @@ def reduce_spec_dataset(tplot_dataset=None, name=None):
                 da = da.sum(dim=d, skipna=True, keep_attrs=True)
     return da
 
-
-def highlight(variables=None, range=None, color='gray', alpha=0.2, fill=True, edgecolor=None, facecolor=None, hatch=None, delete=False):
-    """
-    This function highlights a time interval on tplot variables
-    """
-    if not isinstance(variables, list):
-        variables = [variables]
-
-    for variable in variables:
-        if delete:
-            pytplot.data_quants[variable].attrs['plot_options']['highlight_intervals'] = None
-            continue
-        if range is None:
-            pytplot.data_quants[variable].attrs['plot_options']['highlight_intervals'] = None
-            continue
-        interval = {'location': range,
-                    'color': color,
-                    'alpha': alpha,
-                    'fill': fill,
-                    'edgecolor': edgecolor,
-                    'facecolor': facecolor,
-                    'hatch': hatch}
-        if pytplot.data_quants[variable].attrs['plot_options'].get('highlight_intervals') is None:
-            pytplot.data_quants[variable].attrs['plot_options']['highlight_intervals'] = [interval]
-        else:
-            pytplot.data_quants[variable].attrs['plot_options']['highlight_intervals'].append(interval)
-
-
-def annotate(text=None, position=None,
-             xycoords='figure fraction',
-             color='black',
-             fontfamily=None,
-             fontsize='x-large',
-             alpha=1,
-             fontvariant='normal',
-             fontstyle='normal',
-             fontstretch='normal',
-             fontweight='normal',
-             rotation='horizontal',
-             delete=False):
-    """
-    This function adds annotations to tplot figures
-    """
-    if delete:
-        pytplot.tplot_opt_glob['annotations'] = None
-        return
-
-    annotations = {'text': text,
-                   'position': position,
-                   'xycoords': xycoords,
-                   'fontfamily': fontfamily,
-                   'fontsize': fontsize,
-                   'fontvariant': fontvariant,
-                   'fontstyle': fontstyle,
-                   'fontstretch': fontstretch,
-                   'fontweight': fontweight,
-                   'rotation': rotation,
-                   'color': color,
-                   'alpha': alpha}
-
-    if pytplot.tplot_opt_glob.get('annotations') is None:
-        pytplot.tplot_opt_glob['annotations'] = [annotations]
-    else:
-        pytplot.tplot_opt_glob['annotations'].append(annotations)
\ No newline at end of file
