@@ -16,7 +16,7 @@ def test_math():
 
     pytplot.tplot('s', display=False)
 
-    pytplot.tplot_math.clean_spikes('mvn_euv_calib_bands_x',new_names='despike')
+    pytplot.tplot_math.clean_spikes('mvn_euv_calib_bands_x',newname='despike')
 
     pytplot.tplot('despike',display=False)
 
@@ -40,7 +40,7 @@ def test_math():
 
     pytplot.tplot('data_summed', display=False)
 
-    pytplot.deflag('mvn_euv_calib_bands', 0, new_tvar='deflagged')
+    pytplot.deflag('mvn_euv_calib_bands', 0, newname='deflagged')
 
     pytplot.tplot('deflagged', display=False)
 
@@ -48,7 +48,7 @@ def test_math():
 
     pytplot.tplot('data_flattened', display=False)
 
-    pytplot.join_vec(['mvn_euv_calib_bands_x', 'mvn_euv_calib_bands_y', 'mvn_euv_calib_bands_z'], new_tvar='data2')
+    pytplot.join_vec(['mvn_euv_calib_bands_x', 'mvn_euv_calib_bands_y', 'mvn_euv_calib_bands_z'], newname='data2')
 
     pytplot.tplot('data2', display=False)
 
@@ -64,8 +64,6 @@ def test_math():
 
     pytplot.cdf_to_tplot(os.path.dirname(os.path.realpath(__file__))+ "/testfiles/mvn_swe_l2_svyspec_20170619_v04_r04.cdf")
 
-    pytplot.resample('mvn_euv_calib_bands_y', pytplot.data_quants['diff_en_fluxes'].coords['time'].values, new_tvar='data_3_resampled')
-
     pytplot.tplot('data_3_resampled', display=False)
     pytplot.options('diff_en_fluxes', 'spec', 1)
     pytplot.spec_mult('diff_en_fluxes')
